import math
import numpy as np
from scipy import linalg
from scipy import spatial
import sys

import logging
import matplotlib as mp
import matplotlib.tri as mtri
from mpl_toolkits.mplot3d import Axes3D
from matplotlib.mlab import griddata
import pylab as plt

import IPython

DEF_SDF_FILE = '../Teapot N260113.sdf'
SMALL_SDF_FILE = '../Teapot small N290514.sdf'

class SDF(object):
    def __init__(self, dims, origin, dx, sdf_values):
        self.dims = dims
        self.origin = origin
        self.dx = dx
        self.sdf_values = sdf_values

def read_sdf(filename):
    '''
    Returns a 3d numpy array of SDF values from the input file
    '''
    try:
        f = open(filename, 'r')
    except IOError:
        logging.error('Failed to open sdf file: %s' %(filename))
        return None
    dim_str = f.readline()
    origin_str = f.readline()
    dx_str = f.readline()
    
    # convert header info to floats
    i = 0
    dims = np.zeros([3])
    for d in dim_str.split(' '):
        dims[i] = d
        i = i+1

    i = 0
    origin = np.zeros([3])
    for x in origin_str.split(' '):
        origin[i] = x
        i = i+1

    dx = float(dx_str)

    # read in all sdf values
    sdf_grid = np.zeros(dims)
    i = 0
    j = 0
    k = 0
    for line in f:
        if k < dims[2]:
            sdf_grid[i,j,k] = float(line)
            i = i + 1
            if i == dims[0]:
                i = 0
                j = j+1
            if j == dims[1]:
                j = 0
                k = k+1
    sdf = SDF(dims, origin, dx, sdf_grid)
    return sdf

def render_sdf(sdf, thresh = 0.01):
    h = plt.figure()
    ax = h.add_subplot(111, projection = '3d')

    surface_points = np.where(np.abs(sdf.sdf_values) < thresh)

    x = surface_points[0]
    y = surface_points[1]
    z = surface_points[2]


    xi = np.linspace(min(x), max(x))
    yi = np.linspace(min(y), max(y))
    
    X, Y = np.meshgrid(xi, yi)
    Z = griddata(x, y, z, xi, yi)

    ax.scatter(surface_points[0], surface_points[1], surface_points[2])
#    ax.plot_surface(X, Y, Z, rstride = 6, cstride = 6)
    ax.set_xlabel('X')
    ax.set_ylabel('Y')
    ax.set_zlabel('Z')
    ax.set_xlim3d(0,sdf.dims[0])
    ax.set_ylim3d(0,sdf.dims[1])
    ax.set_zlim3d(0,sdf.dims[2])

    plt.show()

if __name__ == '__main__':
    if len(sys.argv) > 1:
        sdf_filename = sys.argv[1]
    else:
        sdf_filename = DEF_SDF_FILE
    sdf = read_sdf(sdf_filename)
    render_sdf(sdf, 0.05)



from __future__ import print_function

import shlex
import subprocess
import sys

from setuptools import find_packages


version = '0.2.0'


if sys.argv[-1] == 'release':
    # Release via github-actions.
    commands = [
        'git tag v{:s}'.format(version),
        'git push origin main --tag',
    ]
    for cmd in commands:
        print('+ {}'.format(cmd))
        subprocess.check_call(shlex.split(cmd))
    sys.exit(0)

setup_requires = [
]

install_requires = [
    'trimesh>=3.5.20'
]

setup_params = dict(
    name="pysdfgen",
    version=version,
    description="SDFGen for Python",
    author='iory',
    author_email='ab.ioryz@gmail.com',
    url='https://github.com/iory/pySDFGen',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    license="MIT",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    packages=find_packages(include=["pysdfgen", "pysdfgen.*"]),
    package_data={'pysdfgen': ['__init__.py', 'SDFGen']},
    setup_requires=setup_requires,
    install_requires=install_requires,
    include_package_data=False,
)


# https://github.com/skvark/opencv-python/blob/master/setup.py
def install_packages(*requirements):
    # No more convenient way until PEP 518 is implemented;
    # setuptools only handles eggs
    subprocess.check_call(
        [sys.executable, "-m", "pip", "install"] + list(requirements)
    )


# https://github.com/skvark/opencv-python/blob/master/setup.py
def get_or_install_skbuild():
    """If a package is already installed, build against it. If not, install

    """
    # Do not import 3rd-party modules into the current process
    import json
    js_packages = json.loads(
        # valid names & versions are ASCII as per PEP 440
        subprocess.check_output(
            [sys.executable,
             "-m", "pip", "list", "--format", "json",
             "--disable-pip-version-check"]).decode('ascii'))
    if "scikit-build" not in js_packages:
        install_packages("scikit-build!=0.16.0")


def main():
    get_or_install_skbuild()
    import skbuild  # NOQA

    skbuild.setup(**setup_params)


if __name__ == '__main__':
    main()



import os
import os.path as osp
import shutil
import subprocess
import tempfile
import warnings


try:
    from subprocess import DEVNULL
except ImportError:
    DEVNULL = open(os.devnull, 'wb')


_version = None
_SUBMODULES = [
    "obj2sdf",
    "mesh2sdf",
]
__all__ = _SUBMODULES

SDFGen_executable = osp.join(
    osp.abspath(osp.dirname(__file__)), 'SDFGen')

_trimesh = None


def _lazy_trimesh():
    global _trimesh
    if _trimesh is None:
        import trimesh
        _trimesh = trimesh
    return _trimesh


def __getattr__(name):
    global _version
    if name == "__version__":
        if _version is None:
            import pkg_resources
            _version = pkg_resources.get_distribution(
                'pysdfgen').version
        return _version
    raise AttributeError(
        "module {} has no attribute {}".format(__name__, name))


def __dir__():
    return __all__ + ['__version__', '__file__']


def obj2sdf(*args, **kwargs):
    """Convert obj to sdf file.

    Deprecated.
    Use `pysdfgen.mesh2sdf` instead.
    """
    warnings.warn(
        'obj2sdf is deprecated. Use mesh2sdf instead.',
        DeprecationWarning)
    return mesh2sdf(*args, **kwargs)


def mesh2sdf(mesh_filepath, dim=100, padding=5,
             output_filepath=None,
             overwrite=False):
    """Convert mesh file to sdf file.

    Parameters
    ----------
    mesh_filepath : str or pathlib.Path
        filepath of mesh formats that trimesh supports.
    dim : int
        number of sdf dimension.
    padding : int
        number of padding.
    output_filepath : None or str, or pathlib.Path.
        output filepath
    overwrite : bool
        if `True`, overwrite sdf file.
    """
    mesh_filepath = str(mesh_filepath)
    _, ext = osp.splitext(mesh_filepath)

    parent = osp.dirname(mesh_filepath)
    basename = osp.basename(mesh_filepath)
    stem, _ = osp.splitext(basename)
    default_sdf_filepath = osp.join(parent, stem + ".sdf")

    is_obj_file = ext == '.obj'
    if is_obj_file:
        obj_filepath = mesh_filepath
    else:
        # create temporary directory and save obj file.
        tmp_directory = tempfile.mkdtemp()
        tmp_obj_filepath = osp.join(tmp_directory, 'tmp.obj')
        tmp_sdf_filepath = osp.join(tmp_directory, 'tmp.sdf')

        trimesh = _lazy_trimesh()
        mesh = trimesh.load_mesh(mesh_filepath)
        trimesh.exchange.export.export_mesh(mesh, tmp_obj_filepath)
        obj_filepath = tmp_obj_filepath

    if output_filepath is None:
        sdf_filepath = default_sdf_filepath
    else:
        sdf_filepath = output_filepath

    if overwrite is False and osp.exists(sdf_filepath):
        raise OSError("Output file ({}) already exists."
                      .format(sdf_filepath))
    p = subprocess.Popen(
        [SDFGen_executable,
         str(obj_filepath),
         str(dim),
         str(padding)],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)

    stdout, stderr = p.communicate()
    stdout_decoded = stdout.decode('utf-8').strip()
    stderr_decoded = stderr.decode('utf-8').strip()

    if p.returncode != 0:
        error_message = "SDFGen failed."
        if stdout_decoded:
            error_message += " stdout: '{}'".format(stdout_decoded)
        if stderr_decoded:
            error_message += " stderr: '{}'".format(stderr_decoded)
        raise ValueError(error_message)

    if is_obj_file:
        # becuase the output destination of SDFGen can't be specified...
        shutil.move(default_sdf_filepath, sdf_filepath)
    else:
        shutil.move(tmp_sdf_filepath, sdf_filepath)
        shutil.rmtree(tmp_directory)
    return sdf_filepath
